@@ -1,5 +1,5 @@
 from .backends.interface import LockedException
-from .cache_utils import (  # noqa
+from .decorators import (  # noqa
     CacheDetect,
     CircuitBreakerOpen,
     PerfDegradationException,
@@ -7,6 +7,7 @@ from .cache_utils import (  # noqa
     context_cache_detect,
 )
 from .helpers import add_prefix, at  # noqa
+from .validation import set_invalidate_further  # noqa
 from .wrapper import Cache  # noqa
 
 # pylint: disable=invalid-name
