@@ -3,24 +3,31 @@ import datetime
 import gc
 import re
 import sys
+from collections import defaultdict
 from typing import Any, Optional, Tuple, Union
 
+from ..key import get_template_and_func_for
 from .interface import Backend
 
 __all__ = ("Memory", "MemoryInterval")
 
 
 class Memory(Backend):
-    def __init__(self, size: int = 1000):
+    def __init__(self, size: int = 1000, count_stat=False):
         self.store = {}
-        self._chans = []
         self.size = size
+        self._count_stat = count_stat
+        self._counters = defaultdict(lambda: {"hit": 0, "miss": 0, "set": 0})
         self._meta = {}
         self._lock = asyncio.Lock()
         super().__init__()
 
     async def clear(self):
-        self.store = {}
+        async with self._lock:
+            self.store = {}
+            for handler in self._meta.values():
+                handler.cancel()
+            self._meta = {}
 
     async def set(self, key: str, value: Any, expire: Union[None, float, int] = None, exist=None) -> bool:
         if exist is not None:
@@ -49,17 +56,16 @@ class Memory(Backend):
         return value
 
     async def delete(self, key: str):
+        if key not in self.store:
+            return False
         async with self._lock:
             self._delete(key)
 
     def _delete(self, key: str) -> bool:
-        if key in self.store:
-            self._notify("delete", key)
-            del self.store[key]
-            if key in self._meta:
-                del self._meta[key]
-            return True
-        return False
+        del self.store[key]
+        if key in self._meta:
+            del self._meta[key]
+        return True
 
     async def delete_match(self, pattern: str):
         async for key in self.keys_match(pattern):
@@ -69,30 +75,32 @@ class Memory(Backend):
         value = self._get(key)
         if value is None:
             return
-        self._set(key=key, value=value, expire=timeout)
+        self._set_expire_at(key, datetime.datetime.utcnow() + datetime.timedelta(seconds=timeout))
+
+    def _set_expire_at(self, key, date: datetime.datetime):
+        loop = asyncio.get_event_loop()
+        handler = self._meta.get(key)
+        if handler:
+            handler.cancel()
+        self._meta[key] = loop.call_later((date - datetime.datetime.utcnow()).total_seconds(), self._delete, key)
 
     async def get_expire(self, key: str) -> int:
         return -1
 
     async def ping(self, message: Optional[bytes] = None):
-        self._notify("ping", message)
         return b"PONG" if message is None else message
 
     def _set(self, key: str, value: Any, expire: Optional[float] = None):
         if len(self.store) > self.size:
             return
-        self._notify("set", key)
+        self._count_set(key)
         self.store[key] = value
 
         if expire is not None and expire > 0.0:
-            loop = asyncio.get_event_loop()
-            handler = self._meta.get(key)
-            if handler:
-                handler.cancel()
-            self._meta[key] = loop.call_later(expire, self._delete, key)
+            self._set_expire_at(key, datetime.datetime.utcnow() + datetime.timedelta(seconds=expire))
 
     def _get(self, key: str, default: Optional[Any] = None) -> Optional[Any]:
-        self._notify("get", key)
+        self._count_get(key)
         return self.store.get(key, default)
 
     async def set_lock(self, key: str, value, expire):
@@ -116,28 +124,38 @@ class Memory(Backend):
             return _get_obj_size(self.store[key])
         return 0
 
-    def _notify(self, cmd, key):
-        for queue in self._chans:
-            queue.put_nowait((cmd, key))
+    async def get_counters(self, template):
+        if not self._count_stat:
+            raise Exception("Can't get counters: count stat if off")
+        return self._counters.get(template)
 
-    async def listen(self, pattern: str, *cmds, reader=None):
-        queue = asyncio.Queue(maxsize=0)
-        pattern = pattern.replace("*", ".*")
-        regexp = re.compile(pattern)
-        self._chans.append(queue)
-        try:
-            while True:
-                cmd, key = await queue.get()
-                if cmd in cmds and regexp.fullmatch(key):
-                    reader(cmd, key)
-        finally:
-            self._chans.remove(queue)
+    def _count_get(self, key):
+        if not self._count_stat:
+            return
+        template_and_func = get_template_and_func_for(key)
+        if not template_and_func:
+            return
+        template, _ = template_and_func
+
+        if key in self.store:
+            self._counters[template]["hit"] += 1
+        else:
+            self._counters[template]["miss"] += 1
+
+    def _count_set(self, key):
+        if not self._count_stat:
+            return
+        template_and_func = get_template_and_func_for(key)
+        if not template_and_func:
+            return
+        template, _ = template_and_func
+        self._counters[template]["set"] += 1
 
 
 class MemoryInterval(Memory):
-    def __init__(self, size: int = 1000, check_interval: float = 1.0):
+    def __init__(self, size: int = 1000, check_interval: float = 1.0, count_stat=False):
         self._check_interval = check_interval
-        super().__init__(size=size)
+        super().__init__(size=size, count_stat=count_stat)
 
     async def init(self):
         asyncio.create_task(self._remove_expired())
@@ -150,25 +168,32 @@ class MemoryInterval(Memory):
             await asyncio.sleep(self._check_interval)
 
     async def get(self, key: str, default: Optional[Any] = None) -> Optional[Any]:
-        self._notify("get", key)
         expiration = self._meta.get(key)
         if expiration and datetime.datetime.utcnow() > expiration:
             await self.delete(key)
-            return default
+        self._count_get(key)
         return self.store.get(key, default)
 
+    def _set_expire_at(self, key, date: datetime.datetime):
+        self._meta[key] = date
+
     def _set(self, key: str, value: Any, expire: Optional[float] = None) -> None:
-        self._notify("set", key)
+        self._count_set(key)
         self.store[key] = value
 
         if expire is not None and expire > 0.0:
-            self._meta[key] = datetime.datetime.utcnow() + datetime.timedelta(seconds=expire)
+            self._set_expire_at(key, datetime.datetime.utcnow() + datetime.timedelta(seconds=expire))
 
     async def get_expire(self, key: str) -> int:
         if key in self._meta:
-            return abs(int((datetime.datetime.utcnow() - self._meta[key]).total_seconds()))
+            return abs(float((datetime.datetime.utcnow() - self._meta[key]).total_seconds()))
         return -1
 
+    async def clear(self):
+        async with self._lock:
+            self.store = {}
+            self._meta = {}
+
 
 def _get_obj_size(obj) -> int:
     marked = {id(obj)}
