@@ -1,10 +1,11 @@
 import inspect
+import re
 from datetime import timedelta
 from itertools import chain
 from string import Formatter
 from typing import Any, Callable, Dict, Optional, Tuple, Union
 
-from .typing import TTL, FuncArgsType
+from .typing import TTL
 
 
 def ttl_to_seconds(ttl: Union[float, None, TTL]) -> Union[int, None, float]:
@@ -13,11 +14,7 @@ def ttl_to_seconds(ttl: Union[float, None, TTL]) -> Union[int, None, float]:
 
 
 def get_cache_key(
-    func: Callable,
-    template: str = None,
-    args: Tuple[Any] = (),
-    kwargs: Optional[Dict] = None,
-    func_args: FuncArgsType = None,
+    func: Callable, template: Optional[str] = None, args: Tuple[Any] = (), kwargs: Optional[Dict] = None,
 ) -> str:
     """
     Get cache key name for function (:param func) called with args and kwargs
@@ -26,14 +23,13 @@ def get_cache_key(
     :param func: Target function
     :param args: call positional arguments
     :param kwargs: call keyword arguments
-    :param func_args: arguments that will be used in key
     :return: cache key for call
     """
     kwargs = kwargs or {}
-    key_values = get_call_values(func, args, kwargs, func_args)
-    key_values = {k: str(v) if v is not None else "" for k, v in key_values.items()}
-    _key_template = template or get_cache_key_template(func, func_args)
-    return _key_template.format(**key_values).lower()
+    key_values = get_call_values(func, args, kwargs)
+    key_values = {k: v if v is not None else "" for k, v in key_values.items()}
+    _key_template = template or get_cache_key_template(func)
+    return template_to_pattern(_key_template, _formatter=_Blank(""), **key_values).lower()
 
 
 def get_func_params(func):
@@ -43,29 +39,23 @@ def get_func_params(func):
             yield param_name
 
 
-def get_cache_key_template(func: Callable, func_args: FuncArgsType = None, key: Optional[str] = None) -> str:
+def get_cache_key_template(func: Callable, key: Optional[str] = None, prefix: str = "") -> str:
     """
     Get cache key name for function (:param func) called with args and kwargs
-    if func_args is passed key build with parameters are included in func_args dict or tuple otherwise use all of them
     Used function module and name as prefix if key parameter not passed
     :param func: Target function
-    :param args: call positional arguments
-    :param kwargs: call keyword arguments
-    :param func_args: arguments that will be used in key
     :param key: template for key, may contain alias to args or kwargs passed to a call
-    :return: cache key for call
+    :return: cache key template
     """
-    if key:
-        return key
-
-    if func_args is not None:
-        params = {param: "{" + param + "}" for param in func_args}
-    else:
+    if key is None:
         params = {param_name: "{" + param_name + "}" for param_name in get_func_params(func)}
-    return ":".join([func.__module__, func.__name__, *chain(*params.items())]).lower()
+        key = ":".join([func.__module__, func.__name__, *chain(*params.items())]).lower()
+    if prefix:
+        key = f"{prefix}:{key}"
+    return key
 
 
-def get_call_values(func: Callable, args, kwargs, func_args: FuncArgsType) -> Dict:
+def get_call_values(func: Callable, args, kwargs) -> Dict:
     """
     Return dict with arguments and their values for function call with given positional and keywords arguments
     :param func: Target function
@@ -73,14 +63,11 @@ def get_call_values(func: Callable, args, kwargs, func_args: FuncArgsType) -> Di
     :param kwargs: call keyword arguments
     :param func_args: arguments that will be included in results (transformation function for values if passed as dict)
     """
-    key_values = {key: "" for key in func_args or []}
+    key_values = {}
     for _key, _value in _get_call_values(func, args, kwargs).items():
-        if func_args is None or _key in func_args:
-            key_values[_key] = _value
-            if isinstance(func_args, dict) and callable(func_args[_key]):
-                key_values[_key] = func_args[_key](_value)
-            if isinstance(key_values[_key], bytes):
-                key_values[_key] = key_values[_key].decode()
+        key_values[_key] = _value
+        if isinstance(key_values[_key], bytes):
+            key_values[_key] = key_values[_key].decode()
     return key_values
 
 
@@ -99,24 +86,46 @@ def _get_call_values(func, args, kwargs):
     return result
 
 
-class _Star(Formatter):
-    def get_value(self, key, args, kwds):
+class _ReFormatter(Formatter):
+    def get_field(self, field_name, args, kwargs):
         try:
-            return kwds[key]
-        except KeyError:
-            return "*"
+            return super().get_field(field_name, args, kwargs)
+        except (KeyError, AttributeError):
+
+            return f"(?P<{field_name.replace('.', '_')}>.*)", None
 
 
-def template_to_pattern(template, _formatter=_Star(), **values):
-    return _formatter.format(template, **values).lower()
+class _Blank(Formatter):
+    def __init__(self, default="*"):
+        self.__default = default
+        super().__init__()
+
+    def get_field(self, field_name, args, kwargs):
+        try:
+            return super().get_field(field_name, args, kwargs)
+        except (KeyError, AttributeError):
+            return self.__default, None
 
 
-_REGIRSTER = {}
+def template_to_pattern(template: str, _formatter=_Blank(), **values):
+    return _formatter.format(template, **values)
 
 
-def register_template(func, template):
-    _REGIRSTER.setdefault((func.__module__, func.__name__), set()).add(template)
+_REGISTER = {}
+
+
+def register_template(func, template: str):
+    pattern = template_to_pattern(template, _formatter=_ReFormatter())
+    compile_pattern = re.compile(pattern)
+    _REGISTER.setdefault((func.__module__, func.__name__), set()).add((template, compile_pattern))
 
 
 def get_templates_for(func):
-    return _REGIRSTER.get((func.__module__, func.__name__), set())
+    return (template for template, _ in _REGISTER.get((func.__module__, func.__name__), set()))
+
+
+def get_template_and_func_for(key: str):
+    for func, templates in _REGISTER.items():
+        for template, compile_pattern in templates:
+            if compile_pattern.match(key):
+                return template_to_pattern(template), func
