@@ -1,14 +1,15 @@
 import asyncio
+from contextvars import ContextVar
 from functools import partial, wraps
 from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Type, Union
 from urllib.parse import parse_qsl, urlparse
 
-from . import cache_utils
+from . import decorators, validation
 from .backends.interface import Backend, ProxyBackend
 from .backends.memory import Memory, MemoryInterval
 from .helpers import _auto_init, _is_disable_middleware
 from .key import ttl_to_seconds
-from .typing import TTL, FuncArgsType
+from .typing import TTL, CacheCondition
 
 #  pylint: disable=too-many-public-methods
 
@@ -18,19 +19,32 @@ class Cache(ProxyBackend):
         self.__init = False
         self.__address = None
         self._kwargs = {}
-        self._disable: Union[bool, List] = False
-        self.middlewares = (_is_disable_middleware, _auto_init)
+        self.__disable = ContextVar(str(id(self)), default=[])
+        self._set_disable(False)
+        self.middlewares = (_is_disable_middleware, _auto_init, validation._invalidate_middleware)
         super().__init__()
 
+    @property
+    def _disable(self) -> List:
+        return list(self.__disable.get())
+
+    def _set_disable(self, value):
+        if value is True:
+            value = ["cmds", "decorators"]
+        elif value is False:
+            value = []
+        self.__disable.set(value)
+
     @property
     def is_init(self):
         return self.__init
 
     def is_disable(self, *cmds: str) -> bool:
-        if isinstance(self._disable, bool):
-            return self._disable
+        _disable = self._disable
+        if not cmds and _disable:
+            return True
         for cmd in cmds:
-            if cmd.lower() in [c.lower() for c in self._disable]:
+            if cmd.lower() in [c.lower() for c in _disable]:
                 return True
         return False
 
@@ -38,21 +52,22 @@ class Cache(ProxyBackend):
         return not self.is_disable(*cmds)
 
     def disable(self, *cmds: str):
-        if self._disable is True:
-            self._disable = ["cmds", "decorators"]
+        _disable = self._disable
+        if not cmds:
+            _disable = ["cmds", "decorators"]
         if self._disable is False:
-            self._disable = []
-        self._disable.extend(cmds)
+            _disable = []
+        _disable.extend(cmds)
+        self._set_disable(_disable)
 
     def enable(self, *cmds: str):
-        if self._disable is True:
-            self._disable = ["cmds", "decorators"]
-        if self._disable is False:
-            self._disable = []
-            return
+        _disable = self._disable
+        if not cmds:
+            _disable = []
         for cmd in cmds:
-            if cmd in self._disable:
-                self._disable.remove(cmd)
+            if cmd in _disable:
+                _disable.remove(cmd)
+        self._set_disable(_disable)
 
     @property
     def disable_info(self):
@@ -63,12 +78,19 @@ class Cache(ProxyBackend):
         params = settings_url_parse(settings_url)
         params.update(kwargs)
         if "disable" in params:
-            self._disable = params.pop("disable")
+            self._set_disable(params.pop("disable"))
         else:
-            self._disable = not params.pop("enable", True)
-        if not isinstance(self._disable, bool):
-            self._disable = list(self._disable)
+            self._set_disable(not params.pop("enable", True))
+        if "client_side" in params:
+            from .backends.client_side import BcastClientSide, UpdateChannelClientSide
+
+            client_side = params.pop("client_side")
+            params["backend"] = BcastClientSide
+            if client_side == "update":
+                params["backend"] = UpdateChannelClientSide
+
         self._setup_backend(**params)
+        return self
 
     def _setup_backend(self, backend: Type[Backend], **kwargs):
         if self._target:
@@ -155,63 +177,70 @@ class Cache(ProxyBackend):
 
         return _decorator
 
+    def _wrap_on_enable_with_fail_disable(self, name, decorator):
+        def _decorator(func):
+            decorator(func)  # to register cache templates
+
+            @wraps(func)
+            async def _call(*args, **kwargs):
+                if self.is_disable("decorators", name):
+                    return await func(*args, **kwargs)
+                detect = decorators.CacheDetect()
+                result = await decorator(func)(*args, _from_cache=detect, **kwargs)
+                if detect.get():
+                    decorators.context_cache_detect.merge(detect)
+                    self.disable("set")
+                return result
+
+            return _call
+
+        return _decorator
+
     # DecoratorS
     def rate_limit(
         self,
         limit: int,
         period: TTL,
         ttl: Optional[TTL] = None,
-        func_args: FuncArgsType = None,
         action: Optional[Callable] = None,
         prefix="rate_limit",
     ):  # pylint: disable=too-many-arguments
         return self._wrap_on_enable(
             "rate_limit",
-            cache_utils.rate_limit(
-                self,
-                limit=limit,
-                period=ttl_to_seconds(period),
-                ttl=ttl_to_seconds(ttl),
-                func_args=func_args,
-                action=action,
-                prefix=prefix,
+            decorators.rate_limit(
+                self, limit=limit, period=ttl_to_seconds(period), ttl=ttl_to_seconds(ttl), action=action, prefix=prefix,
             ),
         )
 
     def __call__(
-        self,
-        ttl: TTL,
-        func_args: FuncArgsType = None,
-        key: Optional[str] = None,
-        store: Optional[Callable[[Any], bool]] = None,
-        prefix: str = "",
+        self, ttl: TTL, key: Optional[str] = None, condition: CacheCondition = None, prefix: str = "",
     ):
         return self._wrap_on_enable(
             prefix or "cache",
-            cache_utils.cache(self, ttl=ttl_to_seconds(ttl), func_args=func_args, key=key, store=store, prefix=prefix),
+            decorators.cache(self, ttl=ttl_to_seconds(ttl), key=key, condition=condition, prefix=prefix),
         )
 
     cache = __call__
 
     def invalidate(self, target, args_map: Optional[Dict[str, str]] = None, defaults: Optional[Dict] = None):
         return self._wrap_on_enable(
-            "cache", cache_utils.invalidate(self, target=target, args_map=args_map, defaults=defaults)
+            "cache", validation.invalidate(self, target=target, args_map=args_map, defaults=defaults)
         )
 
-    invalidate_func = cache_utils.invalidate_func
+    invalidate_func = validation.invalidate_func
 
     def fail(
         self,
         ttl: TTL,
         exceptions: Union[Type[Exception], Tuple[Type[Exception]]] = Exception,
         key: Optional[str] = None,
-        func_args: FuncArgsType = None,
+        condition: CacheCondition = None,
         prefix: str = "fail",
     ):
-        return self._wrap_on_enable(
+        return self._wrap_on_enable_with_fail_disable(
             prefix,
-            cache_utils.fail(
-                self, ttl=ttl_to_seconds(ttl), exceptions=exceptions, key=key, func_args=func_args, prefix=prefix
+            decorators.fail(
+                self, ttl=ttl_to_seconds(ttl), exceptions=exceptions, key=key, condition=condition, prefix=prefix,
             ),
         )
 
@@ -222,35 +251,27 @@ class Cache(ProxyBackend):
         ttl: TTL,
         exceptions: Union[Type[Exception], Tuple[Type[Exception]]] = Exception,
         key: Optional[str] = None,
-        func_args: FuncArgsType = None,
         prefix: str = "circuit_breaker",
     ):
 
         return self._wrap_on_enable(
             prefix,
-            cache_utils.circuit_breaker(
+            decorators.circuit_breaker(
                 self,
                 errors_rate=errors_rate,
                 period=ttl_to_seconds(period),
                 ttl=ttl_to_seconds(ttl),
                 exceptions=exceptions,
                 key=key,
-                func_args=func_args,
                 prefix=prefix,
             ),
         )
 
     def early(
-        self,
-        ttl: TTL,
-        func_args: FuncArgsType = None,
-        key: Optional[str] = None,
-        store: Optional[Callable[[Any], bool]] = None,
-        prefix: str = "early",
+        self, ttl: TTL, key: Optional[str] = None, condition: CacheCondition = None, prefix: str = "early",
     ):
         return self._wrap_on_enable(
-            prefix,
-            cache_utils.early(self, ttl=ttl_to_seconds(ttl), func_args=func_args, key=key, store=store, prefix=prefix),
+            prefix, decorators.early(self, ttl=ttl_to_seconds(ttl), key=key, condition=condition, prefix=prefix),
         )
 
     def hit(
@@ -258,44 +279,38 @@ class Cache(ProxyBackend):
         ttl: TTL,
         cache_hits: int,
         update_before: int = 0,
-        func_args: FuncArgsType = None,
         key: Optional[str] = None,
-        store: Optional[Callable[[Any], bool]] = None,
+        condition: CacheCondition = None,
         prefix: str = "hit",
     ):
         return self._wrap_on_enable(
             prefix,
-            cache_utils.hit(
+            decorators.hit(
                 self,
                 ttl=ttl_to_seconds(ttl),
                 cache_hits=cache_hits,
                 update_before=update_before,
-                func_args=func_args,
                 key=key,
-                store=store,
+                condition=condition,
                 prefix=prefix,
             ),
         )
 
     def dynamic(
         self,
-        func_args: FuncArgsType = None,
         ttl: TTL = 60 * 60 * 24,
         key: Optional[str] = None,
-        store: Optional[Callable[[Any], bool]] = None,
+        condition: CacheCondition = None,
         prefix: str = "dynamic",
     ):
         return self._wrap_on_enable(
             prefix,
-            cache_utils.hit(
-                self, ttl=ttl, cache_hits=3, update_before=2, func_args=func_args, key=key, store=store, prefix=prefix,
-            ),
+            decorators.hit(self, ttl=ttl, cache_hits=3, update_before=2, key=key, condition=condition, prefix=prefix,),
         )
 
     def perf(
         self,
         ttl: TTL,
-        func_args: FuncArgsType = None,
         key: Optional[str] = None,
         trace_size: int = 10,
         perf_condition: Optional[Callable[[float, Iterable[float]], bool]] = None,
@@ -303,33 +318,24 @@ class Cache(ProxyBackend):
     ):
         return self._wrap_on_enable(
             prefix,
-            cache_utils.perf(
-                self,
-                ttl=ttl,
-                func_args=func_args,
-                key=key,
-                trace_size=trace_size,
-                perf_condition=perf_condition,
-                prefix=prefix,
+            decorators.perf(
+                self, ttl=ttl, key=key, trace_size=trace_size, perf_condition=perf_condition, prefix=prefix,
             ),
         )
 
     def locked(
         self,
         ttl: Optional[TTL] = None,
-        func_args: FuncArgsType = None,
         key: Optional[str] = None,
         step: Union[int, float] = 0.1,
         prefix: str = "locked",
     ):
-        return self._wrap_on_enable(
-            prefix, cache_utils.locked(self, ttl=ttl, func_args=func_args, key=key, step=step, prefix=prefix)
-        )
+        return self._wrap_on_enable(prefix, decorators.locked(self, ttl=ttl, key=key, step=step, prefix=prefix))
 
 
 def _fix_params_types(params: Dict[str, str]) -> Dict[str, Union[str, int, bool, float]]:
     new_params = {}
-    bool_keys = ("safe", "enable", "disable")
+    bool_keys = ("safe", "enable", "disable", "client_side")
     true_values = (
         "1",
         "true",
