@@ -1,3 +1,4 @@
+import asyncio
 from unittest.mock import Mock, patch
 
 import pytest
@@ -15,7 +16,6 @@ def _target():
 @pytest.fixture(name="cache")
 def __cache(target):
     _cache = Cache()
-    _cache._disable = False
     _cache._target = target
     return _cache
 
@@ -50,20 +50,37 @@ async def test_disable_cmd(cache):
     assert await cache.get("test") == 11
 
 
+@pytest.mark.asyncio
+async def test_disable_ctz(cache):
+    await cache.init("mem://localhost")
+    cache.enable("cmds")
+
+    async def test():
+        await cache.set("test", "1")
+        assert await cache.get("test") == "1"
+        cache.disable("cmds")
+        await cache.set("test", "2")
+
+    await asyncio.create_task(test())
+    assert await cache.get("test") == "1"
+    await cache.set("test", "3")
+    assert await cache.get("test") == "3"
+
+
 @pytest.mark.asyncio
 async def test_disable_decorators(cache: Cache, target):
     cache.disable("decorators")
     data = (i for i in range(10))
 
     @cache(ttl=1)
-    # @cache.fail(1)
-    # @cache.hit(ttl=1, cache_hits=1)
-    # @cache.perf(ttl=1)
-    # @cache.circuit_breaker(ttl=1, errors_rate=1, period=1)
-    # @cache.rate_limit(ttl=1, limit=1, period=1)
-    # @cache.early(ttl=1)
-    # @cache.dynamic()
-    # @cache.locked(ttl=1)
+    @cache.fail(1)
+    @cache.hit(ttl=1, cache_hits=1)
+    @cache.perf(ttl=1)
+    @cache.circuit_breaker(ttl=1, errors_rate=1, period=1)
+    @cache.rate_limit(ttl=1, limit=1, period=1)
+    @cache.early(ttl=1)
+    @cache.dynamic()
+    @cache.locked(ttl=1)
     async def func():
         return next(data)
 
@@ -76,6 +93,7 @@ async def test_disable_decorators(cache: Cache, target):
     assert await func() == 2
     target.get.assert_called()
     target.set.assert_called()
+    assert await func() == 2
 
 
 @pytest.mark.asyncio
@@ -145,8 +163,8 @@ async def test_smoke_cmds(cache: Cache, target):
     await cache.get_expire(key="key")  # -> int seconds to expire
     target.get_expire.assert_called_once()
 
-    await cache.ping(message="test")  # -> bytes
-    target.ping.assert_called_once_with(message="test")
+    await cache.ping(message=b"test")  # -> bytes
+    target.ping.assert_called_once_with(message=b"test")
 
     await cache.clear()
     target.clear.assert_called_once_with()
